import { EventAggregator }             from "aurelia-event-aggregator";
import { Router, RouterConfiguration } from "aurelia-router";
import { IAppModelSettings }           from "../../types";
import { PitayaAppViewCustomElement }  from "../components/pitaya-app-view";
import { PitayaView }                  from "./view";

export declare abstract class PitayaApp
{
	public static instance: PitayaApp;
	public viewTransition: string;
	public readonly appView: PitayaAppViewCustomElement;
	public readonly currentView: PitayaView;
	public readonly isOnline: boolean;
	public readonly theme: string;
	public enableControls: boolean;
	protected _router: Router;
	protected _routerConfig: RouterConfiguration;
	protected _settings: IAppModelSettings;

	protected constructor( eventAggregator: EventAggregator );

	public configureRouter( config: RouterConfiguration, router: Router ): Promise<void>;

	public activate( params, routeConfig, navigationInstruction ): Promise<void>;

	public created( parentContext, context );

	public bind( bindingContext, overrideContext );

	public attached();

	public detached();

	public navigate( routeName: string, options?: any ): boolean;

	public navigateToRoute( routeName: string, params?: object, options?: any ): boolean;

	public navigateBack();

	public setStatusBarColor( color: string );
}

export declare function overrideBrowserBackButton( callback: Function );

export declare function setStatusBarColor( color: string );

export declare function navigate( routeName: string, options?: any ): boolean;

export declare function navigateToRoute( routeName: string, params?: object, options?: any ): boolean;

export declare function navigateBack();

export declare function enableControls( enable: boolean );