"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const aurelia_event_aggregator_1 = require("aurelia-event-aggregator");
const aurelia_framework_1 = require("aurelia-framework");
const utils_1 = require("../utility/utils");
const app_configuration_1 = require("./app-configuration");
const view_1 = require("./view");
const _ROUTER_CONFIGURED_CALLBACK_NAME = "routerConfigured";
const _ONLINE_CALLBACK_NAME = "onOnline";
const _OFFLINE_CALLBACK_NAME = "onOffline";
const _APP_VIEW_HTML_ELEMENT_NAME = "pitaya-app-view";
const _TRANSITION_TOP_LEVEL = "pitaya-page-transition--top-level";
const _TRANSITION_TOP_TO_CHILD = "pitaya-page-transition--top-to-child";
const _TRANSITION_CHILD_TO_TOP = "pitaya-page-transition--child-to-top";
const _VIEW_ENTERING_CLASS = "au-enter-active";
const _VIEW_LEAVING_CLASS = "au-leave-active";
const _VIEW_CONTROLS_CLASS = "pitaya-app-view--controls-blocked";
const _META_THEME_COLOR_SELECTOR = "meta[name=theme-color]";
const _PRIMARY_COLOR_CSS_VARIABLE_NAME = "--mdc-theme-primary";
let _currentAppInstance = null;
let _backButtonOverrideCallback = null;
let PitayaApp = class PitayaApp {
    constructor(_eventAggregator) {
        this._eventAggregator = _eventAggregator;
        this._settings = {
            title: "",
            configuration: {},
            routes: null,
            viewTransition: {
                childToTop: {
                    className: _TRANSITION_CHILD_TO_TOP,
                    swapOrder: "with"
                },
                topToChild: {
                    className: _TRANSITION_TOP_TO_CHILD,
                    swapOrder: "with"
                },
                topToTop: {
                    className: _TRANSITION_TOP_LEVEL,
                    swapOrder: "with"
                }
            }
        };
        this._themeColorMetaElement = document.querySelector(_META_THEME_COLOR_SELECTOR);
        this._viewTransition = "";
        this._enableControls = false;
        _currentAppInstance = this;
        _wrapMethod.call(this, this._activate, "activate");
        _wrapMethod.call(this, this._created, "created");
        _wrapMethod.call(this, this._bind, "bind");
    }
    static get instance() {
        return _currentAppInstance;
    }
    get viewTransition() {
        return this._viewTransition;
    }
    set viewTransition(value) {
        const { rootElement } = this._appView;
        const { viewTransition } = this._settings;
        if (!rootElement) {
            return;
        }
        this._viewTransition = value;
        const { classList } = rootElement;
        if (classList.contains(value)) {
            return;
        }
        classList.remove(viewTransition.topToTop.className, viewTransition.topToChild.className, viewTransition.childToTop.className);
        classList.add(value);
    }
    get appView() {
        return this._appView;
    }
    get enableControls() {
        return this._enableControls;
    }
    set enableControls(value) {
        const method = value ? "remove" : "add";
        this._enableControls = value;
        this._appView.rootElement.classList[method](_VIEW_CONTROLS_CLASS);
    }
    get currentView() {
        return view_1.PitayaView.instance || this._router.currentInstruction.viewPortInstructions.default.component.viewModel;
    }
    get isOnline() {
        return this._appView.isOnline;
    }
    get theme() {
        return this._themeColorMetaElement.getAttribute("content");
    }
    async configureRouter(config, router) {
        const routs = await this._getRoutes();
        const [mainRoute] = routs;
        config.title = this._settings.title;
        config.map(routs);
        config.mapUnknownRoutes(mainRoute.moduleId);
        this._router = router;
        this._routerConfig = config;
        this._setRouteChangeHandler(config);
        this[_ROUTER_CONFIGURED_CALLBACK_NAME] && this[_ROUTER_CONFIGURED_CALLBACK_NAME]();
    }
    navigate(routeName, options) {
        return this._router.navigate(routeName, options);
    }
    navigateToRoute(routeName, params, options) {
        return this._router.navigateToRoute(routeName, params, options);
    }
    navigateBack() {
        this._router.navigateBack();
    }
    setStatusBarColor(color) {
        this._themeColorMetaElement.setAttribute("content", color);
    }
    async _activate() {
        await this._initSettings();
    }
    _created(parentContext, context) {
        this._appView = context.controllers.find(controller => controller.behavior.elementName === _APP_VIEW_HTML_ELEMENT_NAME).viewModel;
    }
    _bind() {
        this._initNetworkChangeHandler();
    }
    _setRouteChangeHandler(config) {
        this._eventAggregator.subscribe("router:navigation:processing", this._handleRouteProcessing.bind(this));
        this._eventAggregator.subscribe("router:navigation:success", this._handleRouteSuccess.bind(this));
    }
    _handleRouteProcessing({ instruction }) {
        if (!instruction.previousInstruction) {
            return;
        }
        const { viewTransition } = this._settings;
        const oldOrder = 1 * (instruction.previousInstruction.config.navModel.order || 0);
        const newOrder = 1 * (instruction.config.navModel.order || 0);
        if (oldOrder > newOrder) {
            this.viewTransition = viewTransition.childToTop.className;
            this._appView.routerSwapOrder = viewTransition.childToTop.swapOrder;
        }
        else if (oldOrder < newOrder) {
            this.viewTransition = viewTransition.topToChild.className;
            this._appView.routerSwapOrder = viewTransition.topToChild.swapOrder;
        }
        else {
            this.viewTransition = viewTransition.topToTop.className;
            this._appView.routerSwapOrder = viewTransition.topToTop.swapOrder;
        }
    }
    _handleRouteSuccess(event) {
        this._appView.currentViewName = this._router.currentInstruction.config.name;
    }
    async _initSettings() {
        if (!this._settings.configuration) {
            return;
        }
        let settings;
        if (typeof this._settings.configuration === "string") {
            const file = await fetch(this._settings.configuration);
            settings = await file.json();
        }
        app_configuration_1.PitayaAppConfiguration.set(settings || this._settings.configuration);
    }
    async _getRoutes() {
        if (typeof this._settings.routes === "string") {
            const file = await fetch(this._settings.routes);
            const { routs } = await file.json();
            return routs;
        }
        return this._settings.routes;
    }
    _initNetworkChangeHandler() {
        this._appView.isOnline = navigator.onLine;
        window.addEventListener("online", this._onOnline.bind(this));
        window.addEventListener("offline", this._onOffline.bind(this));
    }
    async _onOnline() {
        this._appView.isOnline = true;
        const { currentView } = this;
        if (_ONLINE_CALLBACK_NAME in currentView) {
            return currentView[_ONLINE_CALLBACK_NAME]();
        }
        (_ONLINE_CALLBACK_NAME in this) && this[_ONLINE_CALLBACK_NAME]();
        await utils_1.delay(100);
        view_1.PitayaView.instance.setTheme();
    }
    async _onOffline() {
        this._appView.isOnline = false;
        const { currentView } = this;
        if (_OFFLINE_CALLBACK_NAME in currentView) {
            return currentView[_OFFLINE_CALLBACK_NAME]();
        }
        (_OFFLINE_CALLBACK_NAME in this) && this[_OFFLINE_CALLBACK_NAME]();
        await utils_1.delay(100);
        view_1.PitayaView.instance.setTheme();
    }
};
PitayaApp = __decorate([
    aurelia_framework_1.inject(aurelia_event_aggregator_1.EventAggregator),
    __metadata("design:paramtypes", [aurelia_event_aggregator_1.EventAggregator])
], PitayaApp);
exports.PitayaApp = PitayaApp;
PitayaApp.prototype["activate"] = function (...args) { };
PitayaApp.prototype["created"] = function (...args) { };
PitayaApp.prototype["bind"] = function (...args) { };
const _noop = () => { };
const _wrapMethod = function (shadowMethod, methodName) {
    const originalMethod = this[methodName] || _noop;
    this.constructor.prototype[methodName] = (...args) => {
        shadowMethod.call(this, ...args);
        originalMethod.call(this, ...args);
    };
};
exports.overrideBrowserBackButton = function (callback) {
    if (_backButtonOverrideCallback) {
        window.removeEventListener("popstate", _backButtonOverrideCallback);
        _backButtonOverrideCallback = null;
    }
    if (!callback) {
        window.history.back();
        return;
    }
    const handler = function () {
        window.history.pushState({}, "");
        callback();
    };
    _backButtonOverrideCallback = handler;
    window.addEventListener("popstate", handler);
    window.history.pushState({}, "");
};
exports.setStatusBarColor = function (color) {
    _currentAppInstance.setStatusBarColor.call(_currentAppInstance, color);
};
exports.navigate = function (routeName, options) {
    return _currentAppInstance.navigate.call(_currentAppInstance, routeName, options);
};
exports.navigateToRoute = function (routeName, params, options) {
    return _currentAppInstance.navigateToRoute.call(_currentAppInstance, routeName, params, options);
};
exports.navigateBack = function () {
    _currentAppInstance.navigateBack.call(_currentAppInstance);
};
exports.enableControls = function (enable = true) {
    _currentAppInstance.enableControls = enable;
};
//# sourceMappingURL=app.js.map