"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _GUID_PREFIX = "pty-";
let _idCounter = 0;
exports.delay = function (ms) {
    return new Promise((resolve) => window.setTimeout(resolve, ms));
};
exports.nextAnimationFrame = function () {
    return new Promise((resolve) => requestAnimationFrame(resolve));
};
exports.nextEventRaise = function (element, eventName) {
    return new Promise((resolve) => {
        const callback = (event) => {
            if (event.srcElement !== element) {
                return;
            }
            element.removeEventListener(eventName, callback);
            resolve();
        };
        element.addEventListener(eventName, callback);
    });
};
exports.createElement = function (htmlString) {
    const template = document.createElement("template");
    template.innerHTML = htmlString.trim();
    return template["content"].firstChild;
};
exports.getJSON = async (url) => {
    const response = await fetch(url);
    const status = response.status;
    if (status >= 200 && status < 300) {
        return response.json();
    }
    else {
        throw {
            status,
            body: await response.json()
        };
    }
};
exports.generateGUID = function () {
    return `${_GUID_PREFIX}${_idCounter++}`;
};
//# sourceMappingURL=utils.js.map